"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.UserMediaState = void 0;

var _userMedia = require("./user-media");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * The goal of this whole thing is to be able to easily get and track available
 * streams and video sources.
 *
 * It starts by checking for permissions because we can do that without
 * triggering any prompts. You can ask to enumerate devices, you just won't get
 * anything useful until you have permission.
 *
 * But in our case we will frequently want the secondary device. So after we get
 * a stream we immediately enumerate devices and try to match it up with the stream.
 */

/**
 * @typedef {  | 'idle' // nothing
 *   | 'start' // starting state
 *   | 'prompt' // browser has not requested permission yet
 *   | 'requesting' // browser has requested permission
 *   | 'denied' // browser has denied permission
 *   | 'granted' // browser has denied permission
 *   | 'requestingDevices'
 *   | 'hasDevices'
 *   | 'unsupported' // browser doesn't support this
 *   | 'hasDevices' // browser has multiple but has not requested it yet
 *   | 'hasStream' // browser has a stream
 *   | 'error' // browser has an error getting stream
 *   | 'streamLost'} MediaState
 */
// This is to avoid build-issues in non-browser environments
const BaseClass = window.EventTarget || class {};

class UserMediaState extends BaseClass {
  constructor() {
    super();
    /** @type {MediaState} */

    this.state = 'idle';
    /** @type {MediaStream | null} */

    this.mediaStream = null;
    /** @type {any[]} */

    this.devices = [];
    this.errorMessage = null;

    this._start();
  }

  _start() {
    // first check permissions
    this._triggerChange('start');

    (0, _userMedia.checkMediaPermissions)().then(state => {
      // in any one of these states, we can't do anything
      if (state === 'unsupported' || state === 'denied' || state === 'prompt') {
        this._triggerChange(state);

        return;
      } else {
        // if we're granted, we can get the devices
        return this.getDevices();
      }
    });
  }
  /** @param {MediaState} newState */


  _triggerChange(newState) {
    if (this.state === newState) {
      return;
    }

    if (this.errorMessage && newState !== 'error') {
      this.errorMessage = null;
    }

    const previousState = this.state;
    this.state = newState;
    this.dispatchEvent(new CustomEvent('statechange', {
      detail: {
        state: newState,
        previousState
      }
    }));
  }

  _clearExistingStream() {
    if (this.mediaStream) {
      const stream = this.mediaStream;
      this.mediaStream = null;
      (0, _userMedia.stopStream)(stream);
      return stream;
    }
  }

  _streamLost() {
    if (this.mediaStream) {
      this.dispatchEvent(new CustomEvent('streamLost', {
        detail: {
          stream: this._clearExistingStream()
        }
      }));
    }
  }

  _handleGotStream(stream) {
    this._triggerChange('hasStream');

    this._clearExistingStream();

    this.mediaStream = stream;
    const firstVideoTrack = stream.getVideoTracks()[0];
    const {
      label,
      fakeLabel
    } = firstVideoTrack; // console.log('firstVideoTrack', firstVideoTrack)

    const settings = firstVideoTrack.getSettings(); // console.log('settings', settings)

    const {
      width,
      height,
      deviceId
    } = settings;
    this.dispatchEvent(new CustomEvent('stream', {
      detail: {
        stream,
        streamId: stream.id,
        deviceId,
        label: fakeLabel || label,
        width,
        height
      }
    }));
    stream.addEventListener('inactive', () => {
      this._streamLost();

      this.getDevices();
    });
  }

  requestStream(deviceId, {
    width: idealWidth = 800,
    height: idealHeight = 600
  } = {}) {
    this._clearExistingStream();

    this._triggerChange('requesting');

    const constraints = {
      audio: false,
      video: {
        width: {
          ideal: idealWidth
        },
        height: {
          ideal: idealHeight
        }
      }
    };

    if (deviceId) {
      constraints.video.deviceId = {
        exact: deviceId
      };
    } else {
      constraints.video.facingMode = 'environment';
    }

    return (0, _userMedia.getStream)(deviceId, constraints).then(stream => this.getDevices(stream).then(() => {
      this._handleGotStream(stream);

      return stream;
    })).catch(err => {
      const isNotAllowedError = err.name === 'NotAllowedError';
      const errorMessage = {
        NotAllowedError: 'User denied access',
        AbortError: 'Device access aborted',
        NotFoundError: 'Not matching devices found',
        NotReadableError: 'Selected device not readable',
        OverconstrainedError: 'Impossible constraints passed',
        SecurityError: 'Security error encountered',
        TypeError: 'Invalid constraints'
      }[err.name];

      if (isNotAllowedError) {
        this._triggerChange('denied');
      } else {
        this.errorMessage = errorMessage || err.message;

        this._triggerChange('error');
      }

      throw err;
    });
  }

  getDevices(stream) {
    this._triggerChange('requestingDevices');

    return (0, _userMedia.getDevices)({
      types: ['videoinput']
    }).then(devices => {
      // if we were passed a stream, try to match it up with a device
      let selectedLabel;

      if (stream) {
        const firstVideoTrack = stream.getVideoTracks()[0];
        selectedLabel = firstVideoTrack.fakeLabel || firstVideoTrack.label;
      }

      this.devices = devices.map(device => _objectSpread(_objectSpread({}, device), {}, {
        selected: device.label === selectedLabel
      }));
      this.dispatchEvent(new CustomEvent('devicesChange', {
        detail: devices
      }));

      this._triggerChange('hasDevices');

      return devices;
    });
  }

}

exports.UserMediaState = UserMediaState;